@@ -57,7 +57,7 @@
 
 #include <sqlite3.h>
 
-#define MINIDLNA_VERSION "1.1.1"
+#define MINIDLNA_VERSION "1.1.3"
 
 #ifdef NETGEAR
 # define SERVER_NAME "ReadyDLNA"
@@ -73,6 +73,7 @@
 #else
 #define _(string) (string)
 #endif
+#define THISORNUL(s) (s ? s : "")
 
 #ifndef PNPX
 #define PNPX 0
@@ -189,6 +190,7 @@ extern uint32_t runtime_flags;
 #define DLNA_STRICT_MASK      0x0004
 #define NO_PLAYLIST_MASK      0x0008
 #define SYSTEMD_MASK          0x0010
+#define MERGE_MEDIA_DIRS_MASK 0x0020
 
 #define SETFLAG(mask)	runtime_flags |= mask
 #define GETFLAG(mask)	(runtime_flags & mask)
@@ -215,12 +217,11 @@ extern char pnpx_hwid[];
 #endif
 
 /* lan addresses */
-/* MAX_LAN_ADDR : maximum number of interfaces
- * to listen to SSDP traffic */
 extern int n_lan_addr;
 extern struct lan_addr_s lan_addr[];
+extern int sssdp;
 
-extern const char * minissdpdsocketpath;
+extern const char *minissdpdsocketpath;
 
 /* UPnP-A/V [DLNA] */
 extern sqlite3 *db;
@@ -233,5 +234,6 @@ extern struct album_art_name_s *album_art_names;
 extern short int scanning;
 extern volatile short int quitting;
 extern volatile uint32_t updateID;
+extern const char *force_sort_criteria;
 
 #endif
